require "import"
import "android.content.Context"
import "android.content.Intent"
import "android.os.Handler"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "java.util.Locale"
import "android.net.Uri"
import "android.widget.LinearLayout"
import "android.widget.TextView"

-- Variable to hold the current language
local currentLanguage = Locale.getDefault().toString() -- Default language is set to system locale

-- Function to set the language dynamically
function setLanguage(languageCode)
    currentLanguage = languageCode
end

-- Function to check for internet connection
function hasInternetConnection()
    local connectivityManager = this.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

-- Function to stop speech recognition
function kill()
    if speechRecognizer then
        speechRecognizer.destroy()
        speechRecognizer = nil
    end
end

-- Function to start listening for speech recognition
function startListening()
    if not hasInternetConnection() then
        service.asyncSpeak("Please check your internet connection and try again.")
        return false
    end

    local verbalizationDetected = false

    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0):lower()

                -- Check for language switching commands
                if recognizedText:find("tamil") then
                    service.speak("Switching to Tamil")
                    setLanguage("ta-IN") -- Tamil locale
                elseif recognizedText:find("kannada") then
                    service.speak("Switching to Kannada")
                    setLanguage("kn-IN") -- Kannada locale
                elseif recognizedText:find("malayalam") then
                    service.speak("Switching to Malayalam")
                    setLanguage("ml-IN") -- Malayalam locale
                elseif recognizedText:find("english") then
                    service.speak("Switching to English")
                    setLanguage("en-IN") -- English locale
                else
                    -- Insert recognized text into the edit box
                    local node = service.getEditText()
                    service.insertText(node, recognizedText)
                    service.speak("You said: " .. recognizedText)
                end

                -- Stop and restart speech recognition after the language switch
                service.handler.postDelayed(function()
                    startListening() -- Restart speech recognition
                end, 2000)
            end

            if not verbalizationDetected then
                kill()
                return false
            end
        end,
        onError = function()
            kill()
            return false
        end
    }

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, currentLanguage) -- Set dynamically chosen language
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(speechListener)

    return true
end

-- Check if the dialog has been shown before
local prefs = this.getSharedPreferences("MyPrefs", Context.MODE_PRIVATE)
local firstRun = prefs.getBoolean("firstRun", true)

if firstRun then
    -- Show the initial dialog only once
    local layout = LinearLayout(this)
    layout.setOrientation(LinearLayout.VERTICAL)

    local dlg = LuaDialog(service)
    dlg.setButton("Connect on Telegram", function()
        local intent = Intent(Intent.ACTION_VIEW, Uri.parse("https://t.me/jieshuofiles"))
        this.startActivity(intent)
    end)

    this.postSpeak(2000, "Modded by R. D. S.! This dialog will only show once.")
    dlg.setTitle("Voice Dictation")
    dlg.View = layout
    dlg.show()

    -- Mark that the dialog has been shown
    local editor = prefs.edit()
    editor.putBoolean("firstRun", false)
    editor.apply()
else
    -- Start speech recognition after the first run
    startListening()
end